/*****************************************
 Copyright  2001-2003	
 Sigma Designs, Inc. All Rights Reserved
 Proprietary and Confidential
 *****************************************/

/*****************************************************************************/

/*
 *      realmagicdev.c -- NetStream2000 EM8400 (quasar) ``major level'' driver
 *
 *      Copyright (C) 1999-2000  Sigma Designs
 *                    written by Emmanuel Michon <emmanuel_michon@sdesigns.com>
 *                           and Howard G. Page
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*****************************************************************************/

#include <linux/init.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/pci.h>
#include <linux/fs.h>
#include <asm/io.h>

#include "quasar.h"
#include "realmagicdev.h"

MODULE_LICENSE("GPL");

// this table stores up to REALMAGIC_MAX_DEV devices
realmagic_dev_t realmagic_dev_tab[REALMAGIC_MAX_DEV];
int realmagic_dev_idx;

// look in linux/fs.h for types
static ssize_t realmagic_read(struct file *, char *, size_t, loff_t *);
static int realmagic_ioctl(struct inode *, struct file *, unsigned int, unsigned long);
static int realmagic_mmap(struct file *, struct vm_area_struct *);
static int realmagic_open(struct inode *, struct file *);
static int realmagic_release(struct inode *, struct file *);

// override this default by using: ``insmod anyrealmagic.o quasar_dev_id=0x8450'' for instance.
#ifdef _STREAMMACHINE_
unsigned short quasar_dev_id=0x8475; 
#else
unsigned short quasar_dev_id=0x8400; 
#endif /*_STREAMMACHINE_*/

module_param(quasar_dev_id, short, 0);

struct file_operations realmagic_fops = 
{
	.owner = 	THIS_MODULE,
	.read = 	realmagic_read, 
	.ioctl = 	realmagic_ioctl, 
	.mmap = 	realmagic_mmap, 
	.open = 	realmagic_open, 
	.release = 	realmagic_release, 
};

// extract minor number when accessed by struct inode
int realmagic_get_dev(struct inode *i_node,struct file_operations **fops)
{
	int idx = MINOR(i_node->i_rdev);
	realmagic_dev_t *dev;
	dev=&realmagic_dev_tab[idx];
	*fops=dev->fops;
	return 0;
}

// extract minor number when accessed by struct file
int realmagic_get_dev_file(struct file *file, struct file_operations **fops)
{
	return realmagic_get_dev(file->f_dentry->d_inode,fops);
}

static ssize_t realmagic_read(struct file *filp, char *buf, size_t count, loff_t *pos)
{
	struct file_operations *dev_fops=0;
	realmagic_get_dev_file(filp,&dev_fops);
	return (*dev_fops->read)(filp,buf,count,pos);
}

static int realmagic_ioctl(struct inode* i_node, struct file* filp,
			   unsigned int arg1, unsigned long arg2)
{
	struct file_operations *dev_fops=0;
	realmagic_get_dev(i_node,&dev_fops);
	(*dev_fops->ioctl)(i_node, filp, arg1, arg2);
	return 0;
}

static int realmagic_mmap(struct file* filp,
			  struct vm_area_struct* vmptr)
{
	struct file_operations *dev_fops=0;
	realmagic_get_dev_file(filp,&dev_fops);
	(*dev_fops->mmap)(filp, vmptr);
	return 0;
}

static int realmagic_open(struct inode *i_node, struct file *filp)
{
	struct file_operations *dev_fops=0;
	realmagic_get_dev(i_node,&dev_fops);
	(*dev_fops->open)(i_node, filp);
	return 0;
}

static int realmagic_release(struct inode *i_node, struct file *filp)
{
	struct file_operations *dev_fops=0;
	realmagic_get_dev(i_node,&dev_fops);
	(*dev_fops->release)(i_node, filp);
	return 0;
}

static int __init realmagic84xx_init(void)
{
	realmagic_dev_idx=0;
	// look for hardware
	quasar_init();
	// register char devices
	register_chrdev(REALMAGIC_MAJOR, REALMAGIC_DEVICE_NAME, &realmagic_fops);
	printk("init_module success \n");
	return 0;
}

int cleanup_module_with_return_code(void)
{
	quasar_cleanup();
	unregister_chrdev(REALMAGIC_MAJOR, REALMAGIC_DEVICE_NAME);
	return 0;
}

static void __exit realmagic84xx_exit(void)
{
	int rc;
	rc=cleanup_module_with_return_code();
	printk("cleanup_module over with %d \n",rc);
}

module_init(realmagic84xx_init);
module_exit(realmagic84xx_exit);
